@@ -12,8 +12,70 @@ from rich.table import Table
 from rich.text import Text
 from rich.console import Console
 from rich import box
-from .operators.eval_operator import trigger_eval_job, display_evaluation_status
+from .operators.eval_operator import trigger_eval_job, display_evaluation_status, trigger_eval_sweep, eval_log_print
+from typing import Optional, Set, Tuple
+import re
 
+_ALLOWED_KEYS = {"task", "tp_size", "num_fewshot", "batch_size", "seq_length", "dp_size"}
+_TASK_NAME_RE = re.compile(r"^[A-Za-z0-9_.:\-\/]+$")
+
+def _validate_task_and_reason(task: str, allowed_tasks: Optional[Set[str]] = None) -> Tuple[bool, str]:
+    if task is None:
+        return False, "empty input"
+
+    s = task.strip()
+    if not s:
+        return False, "empty input"
+    
+    if "=" not in s:
+        parts = [p.strip() for p in s.split("|")]
+        if not parts or any(not p for p in parts):
+            return False, "empty task in basic mode"
+        for t in parts:
+            if not _TASK_NAME_RE.match(t):
+                return False, f"invalid task name: {t}"
+            if allowed_tasks is not None and t not in allowed_tasks:
+                return False, f"task not allowed: {t}"
+        return True, "basic"
+    
+    groups = [g.strip() for g in s.split("|")]
+    if not groups or any(not g for g in groups):
+        return False, "empty group in advanced mode"
+
+    for grp in groups:
+        kvs = [p.strip() for p in grp.split(",") if p.strip()]
+        if not kvs:
+            return False, "empty kv list in group"
+
+        seen = {}
+        for kv in kvs:
+            if "=" not in kv:
+                return False, f"missing '=' in '{kv}'"
+            k, v = kv.split("=", 1)
+            k, v = k.strip(), v.strip()
+
+            if k not in _ALLOWED_KEYS:
+                return False, f"unknown key '{k}'"
+            if k in seen:
+                return False, f"duplicate key '{k}'"
+            if k == "task":
+                if not v:
+                    return False, "empty task value"
+                if not _TASK_NAME_RE.match(v):
+                    return False, f"invalid task name: {v}"
+                if allowed_tasks is not None and v not in allowed_tasks:
+                    return False, f"task not allowed: {v}"
+            else:
+                if not re.fullmatch(r"\d+", v):
+                    return False, f"value for '{k}' must be a non-negative integer: {v}"
+
+            seen[k] = v
+
+        if "task" not in seen:
+            return False, "missing required key 'task' in group"
+    
+    return True, "advanced"
+    
 @click_group()
 def eval():
     pass
@@ -393,9 +455,20 @@ def model(ctx,
     config_table.add_row("Project Name", project_name)
     
     # Display dataset tasks if provided
+    
     if dataset_tasks:
+        valid, reason = _validate_task_and_reason(dataset_tasks)
+        if not valid:
+            print(f"❌ Invalid dataset tasks: {reason}")
+            sys.exit(1)
+        
         # Split dataset_tasks by "|" and display each task on a separate line
         tasks_list = [task.strip() for task in dataset_tasks.split("|")]
+        for task in tasks_list:
+            if task.startswith(";"):
+                tasks_list.remove(task)
+                tasks_list.insert(0, task)
+        
         config_table.add_row("Dataset Tasks Mode", "List Mode")
         config_table.add_row("Total Tasks", f"{len(tasks_list)} tasks")
         for i, task in enumerate(tasks_list, 1):
@@ -476,5 +549,24 @@ def status(eval_id):
    """check eval job: anc eval status xxx """
    display_evaluation_status(eval_id)
 
+
+@eval.command()
+@click.argument('spec', required=True, type=str)
+@click.pass_context
+def sweep(ctx, 
+          spec):
+    project = os.getenv("MLP_PROJECT", "llm")
+    cluster = os.getenv("MLP_CLUSTER", "il2")
+    trigger_eval_sweep(spec, cluster, project)
+
+@eval.command()
+@click.argument('evalution_id', required=True, type=str)
+@click.pass_context
+def log(ctx, 
+          evalution_id):
+    project = os.getenv("MLP_PROJECT", "llm")
+    cluster = os.getenv("MLP_CLUSTER", "il2")
+    eval_log_print(evalution_id, cluster)
+
 def add_command(cli_group):
-    cli_group.add_command(eval)
\ No newline at end of file
+    cli_group.add_command(eval)
