@@ -3,6 +3,7 @@ from anc.data.anc_composer import SeqSplitConfig
 from anc.data.processors.tokenizer import HFPretrainedTokenizer
 import logging
 import os
+import sys
 import torch
 import json
 from typing import List, Tuple
@@ -154,7 +155,7 @@ class PretrainProcessor(Processor):
             return None
         tokens = self.tokenizer.txt2vec(item['content'])
         if len(tokens) > self.max_seq_len and not self.seq_split_config.allow_split:
-            logging.warn(f"encounter sample with token length larger than {self.max_seq_len}, will ignore and continue")
+            logging.warning(f"pretrain sample encounter with token length larger than {self.max_seq_len}, will ignore and continue")
             return None
         loss_mask = [1] * len(tokens)
         # set mask between two special tokens <MASK:Z0k7RxyNaZ> and </MASK:Z0k7RxyNaZ> to zero
@@ -487,6 +488,8 @@ class SFTProcessor(PretrainProcessor):
         pad_label_id=None,
         cu_seqlen_use_old_pack=False,
         pad_to_max_seq_len=True,
+        special_tokens=None,
+        special_token_ids=None,
     ):
         """Initialize SFT processor with model and processing configurations.
         
@@ -511,6 +514,8 @@ class SFTProcessor(PretrainProcessor):
             pad_label_id,
             cu_seqlen_use_old_pack,
             pad_to_max_seq_len=pad_to_max_seq_len,
+            special_tokens=special_tokens,
+            special_token_ids=special_token_ids,
         )
         self.opt = FakeOpt(hf_model_name, prompt_style, add_bos)
         setup_separate_prompt(self.opt)
@@ -525,6 +530,18 @@ class SFTProcessor(PretrainProcessor):
         Yields:
             dict: Processed item
         """
+        # if this is a pretrain data, delegate to pretrain processor
+        if 'text' in item:
+            pretrain_data = {"content": item['text']}
+            if 'id' in item:
+                pretrain_data['id'] = item['id']
+            if 'filepath' in item:
+                pretrain_data['filepath'] = item['filepath']
+            if 'row_idx' in item:
+                pretrain_data['row_idx'] = item['row_idx']
+            yield from super().transform(pretrain_data, is_last_sample)
+            return
+
         # Validate that all messages have required fields with correct types
         if not all(turn['role'] and turn['content'] and 
                   type(turn['role']) is str and type(turn['content']) is str 
@@ -532,7 +549,7 @@ class SFTProcessor(PretrainProcessor):
             return
             
         # Process conversation context
-        context = [(turn['role'].strip(), turn['content']) for turn in item['messages']]
+        context = [(turn['role'].strip(), turn['content'], turn.get('no_loss', False)) for turn in item['messages']]
         last_turn_mask_flag = item.get('last_turn_mask', False) or item.get('loss_on_last_turn', False)
         no_mask_flag = item.get('no_mask', False)
         
@@ -560,7 +577,7 @@ class SFTProcessor(PretrainProcessor):
             context,
             self.opt.delimiter,
             loss_on_last_turn=last_turn_mask_flag,
-            no_query_mask=no_mask_flag,
+            enable_query_mask=no_mask_flag,
         )
         self.try_add_bos_token(context_vec, loss_mask)
         return context_vec, loss_mask
@@ -570,7 +587,7 @@ class SFTProcessor(PretrainProcessor):
         context: List[Tuple[str, str]], 
         dialog_sep: str=None, 
         loss_on_last_turn=False, 
-        no_query_mask=False, 
+        enable_query_mask=False, 
         add_generation_suffix=False,
         **kwargs
     ):
@@ -580,13 +597,13 @@ class SFTProcessor(PretrainProcessor):
             context: List of (role, content) tuples for the conversation
             dialog_sep: Separator between dialogue turns
             loss_on_last_turn: Whether to only compute loss on the last turn
-            no_query_mask: Whether to disable masking for queries
+            enable_query_mask: Whether to disable masking for queries
             add_generation_suffix: Whether to add assistant prefix for generation
             
         Returns:
             tuple: (token_ids, loss_mask) for the formatted conversation
         """
-        assert not (loss_on_last_turn and no_query_mask)
+        assert not (loss_on_last_turn and enable_query_mask)
         if dialog_sep is None:
             dialog_sep: str = self.opt.delimiter
 
@@ -602,7 +619,7 @@ class SFTProcessor(PretrainProcessor):
             Returns:
                 tuple: (role_tokens, content_tokens, suffix_tokens)
             """
-            if is_output or no_query_mask:
+            if is_output or enable_query_mask:
                 # For assistant outputs or when no_query_mask is True,
                 # encode role prefix separately from content
                 encoded_role, encoded_content, encoded_suffix = (
@@ -625,9 +642,9 @@ class SFTProcessor(PretrainProcessor):
         do_capitalize = self.opt.do_capitalize
 
         # Process each turn in the conversation
-        for i, (role, content) in enumerate(context):
+        for i, (role, content, no_loss) in enumerate(context):
             suffix_sep = dialog_sep
-                
+
             if role == 'assistant':
                 if do_capitalize:
                     role = role.capitalize()
@@ -641,7 +658,11 @@ class SFTProcessor(PretrainProcessor):
                     is_output=True
                 )
                 # Set mask based on loss_on_last_turn flag
-                content_mask = [int(not loss_on_last_turn or i == len(context) - 1)] * len(encoded_content)
+                if loss_on_last_turn:
+                    to_learn = i == len(context) - 1
+                else:
+                    to_learn = not no_loss
+                content_mask = [int(to_learn)] * len(encoded_content)
             else:
                 # Handle other roles (user, system, etc.)
                 if role not in ('user', 'human', 'system', 'document', 'memory', 
@@ -654,7 +675,7 @@ class SFTProcessor(PretrainProcessor):
                     content, 
                     suffix_sep
                 )
-                content_mask = [int(no_query_mask)] * len(encoded_content)
+                content_mask = [int(enable_query_mask)] * len(encoded_content)
 
             # Combine encoded parts and their masks
             idxs_encoded.extend(encoded_role + encoded_content + encoded_suffix)
